/*
 * Copyright (c) 2020 UPLEX Nils Goroll Systemoptimierung
 * All rights reserved
 *
 * Author: Geoffrey Simmons <geoffrey.simmons@uplex.de>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

package vcl

import "text/template"

const rewriteTmplSrc = `
import re2;
import selector;

{{range $i, $r := .Rewrites -}}
{{if needsMatcher $r -}}
sub vcl_init {
	new {{rewrName $i}} = {{vmod $r.Compare}}.set({{rewrFlags $r}});
	{{- range $rule := $r.Rules}}
	{{rewrName $i}}.add("{{$rule.Value}}", string="{{$rule.Rewrite}}"
		{{- if needsSave $r}}, save=true{{end -}}
		{{- if needsRegex $r}}, regex="{{saveRegex $r $rule}}"{{end -}}
		{{- if needsNeverCapture $r}}, never_capture=true{{end -}}
		);
        {{- end}}
        {{- if needsCompile $r.Compare}}
	{{rewrName $i}}.compile();
	{{- end}}
}
{{- end}}

sub vcl_{{rewrSub $r}} {
	{{- if needsMatcher $r}}
	if ({{rewrName $i}}.{{match $r.Compare}}({{$r.Source}})) {
		{{- if needsUniqueCheck $r}}
		if ({{rewrName $i}}.nmatches() != 1) {
			std.log({{$r.Source}} + " had " +
				{{rewrName $i}}.nmatches() + " matches");
			return(fail);
		}
		{{- end}}
        {{- end}}
	{{- if rewrMethodDelete $r}}
		unset {{$r.Target}};
	{{- else if rewrMethodAppend $r}}
		set {{$r.Target}} = {{rewrOperand1 $r}} + {{rewrOperand2 $r $i}};
	{{- else if rewrMethodPrepend $r}}
		set {{$r.Target}} = {{rewrOperand2 $r $i}} + {{rewrOperand1 $r}};
	{{- else if rewrMethodReplace $r}}
		set {{$r.Target}} = {{rewrOperand2 $r $i}};
	{{- else}}
		set {{$r.Target}} =
			{{rewrName $i}}.{{rewrOp $r}}({{$r.Source}},
		                       {{rewrName $i}}.string({{rewrSelect $r}})
		                       {{- if needsAll $r}}, all=true{{end -}}
		                       {{- if needsSelectEnum $r -}}
                                       , {{rewrSelect $r}}{{end -}}
			);
	{{- end}}
	{{- if needsMatcher $r}}
	}
	{{- end}}
}
{{- end}}
`

const rewriteTmplName = "rewrite"

var rewriteTmpl = template.Must(template.New(rewriteTmplName).Funcs(fMap).
	Parse(rewriteTmplSrc))
